﻿using System;
using System.Linq;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Activation;
using Microsoft.AnalysisServices.AdomdClient;
using System.ComponentModel;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Xml.Serialization;

using Dimension = CustomDataProvider.Web.Framework.AdomdDimension;

namespace CustomDataProvider.Web
{
    using System.Data;
    using System.Text.RegularExpressions;

    using Measure = CustomDataProvider.Web.Framework.AdomdMeasure;
    using Member = CustomDataProvider.Web.Framework.AdomdMember;
    using CustomDataProvider.Web.Framework;

    [ServiceContract(Namespace = "")]
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public class AdomdConnector
    {

        [OperationContract]
        public string LoadSchema(string connectionString)
        {
            Microsoft.AnalysisServices.AdomdClient.AdomdConnection conn =
                new Microsoft.AnalysisServices.AdomdClient.AdomdConnection(connectionString);

            conn.Open();

            return conn.Database;
        }

        [OperationContract]
        public ObservableCollection<AdomdCube> LoadCubes(string connectionString)
        {
            Microsoft.AnalysisServices.AdomdClient.AdomdConnection conn =
                new Microsoft.AnalysisServices.AdomdClient.AdomdConnection(connectionString);

            conn.Open();

            ObservableCollection<AdomdCube> cubeDefsCollection = new ObservableCollection<AdomdCube>();
            foreach (CubeDef cube in conn.Cubes)
            {
                if (cube.Type == Microsoft.AnalysisServices.AdomdClient.CubeType.Cube)
                {
                    cubeDefsCollection.Add(
                        new AdomdCube
                        {
                            CubeName = cube.Name,
                            Caption = cube.Caption,
                            Description = cube.Description,
                            LastDataUpdate = cube.LastUpdated,
                            CubeType = cube.Type.ToString()
                        });
                }
            }

            return cubeDefsCollection;
        }

        [OperationContract]
        public ObservableCollection<Dimension> LoadDimensions(string connectionString, string cubeName)
        {
            Microsoft.AnalysisServices.AdomdClient.AdomdConnection conn =
                new Microsoft.AnalysisServices.AdomdClient.AdomdConnection(connectionString);

            conn.Open();


            CubeDef cubeDef = conn.Cubes[cubeName];

            if (cubeDef == null)
            {
                return null;
            }

            ObservableCollection<Dimension> dimensions =
                new ObservableCollection<Dimension>();

            foreach (Microsoft.AnalysisServices.AdomdClient.Dimension dimensionData in cubeDef.Dimensions)
            {
                if (dimensionData.DimensionType == DimensionTypeEnum.Measure)
                {
                    continue;
                }

                Dimension currentDimension =
                    new Dimension()
                    {
                        Name = dimensionData.Name,
                        UniqueName = dimensionData.UniqueName,
                        Caption = dimensionData.Caption,
                        Description = dimensionData.Description,
                        DimensionType = dimensionData.DimensionType.ToString()
                    };

                foreach (Microsoft.AnalysisServices.AdomdClient.Hierarchy hierarchyData in dimensionData.Hierarchies)
                {
                    AdomdHierarchy currentHierarchy =
                        new AdomdHierarchy()
                        {
                            Name = hierarchyData.Name,
                            UniqueName = hierarchyData.UniqueName,
                            Caption = hierarchyData.Caption,
                            Description = hierarchyData.Description,
                            DefaultMember = hierarchyData.DefaultMember,
                            DimensionUniqueName = currentDimension.UniqueName
                        };

                    foreach (Microsoft.AnalysisServices.AdomdClient.Level level in hierarchyData.Levels)
                    {
                        AdomdLevel currentLevel =
                        new AdomdLevel()
                        {
                            Name = level.Name,
                            Caption = level.Caption,
                            UniqueName = level.UniqueName,
                            Description = level.Description,
                            Number = (uint)level.LevelNumber,
                            HierarchyUniqueName = currentHierarchy.Name
                        };

                        currentHierarchy.Levels.Add(currentLevel);
                    }

                    currentDimension.Hierarchies.Add(currentHierarchy);
                }

                dimensions.Add(currentDimension);
            }

            return dimensions;
        }

        [OperationContract]
        public ObservableCollection<Measure> LoadMeasures(string connectionString, string cubeName)
        {
            Microsoft.AnalysisServices.AdomdClient.AdomdConnection conn =
                new Microsoft.AnalysisServices.AdomdClient.AdomdConnection(connectionString);

            conn.Open();

            CubeDef cubeDef = conn.Cubes[cubeName];

            if (cubeDef == null)
            {
                return null;
            }

            ObservableCollection<Measure> mesures = new ObservableCollection<Measure>();

            foreach (Microsoft.AnalysisServices.AdomdClient.Measure measure in cubeDef.Measures)
            {
                string measureGroupName = measure.Properties["MEASUREGROUP_NAME"].Value.ToString();
                string aggregator = measure.Properties["MEASURE_AGGREGATOR"].Value.ToString();

                AdomdMeasure currentMeasure = new Measure
                {
                    Name = measure.Name,
                    UniqueName = measure.UniqueName,
                    Caption = measure.Caption,
                    Description = measure.Description,
                    MeasureGroupName = measureGroupName,
                    Aggregator = aggregator
                };

                mesures.Add(currentMeasure);
            }

            return mesures;
        }

        [OperationContract]
        public List<string> LoadMeasureGroups(string connectionString, string cubeName)
        {
            Microsoft.AnalysisServices.AdomdClient.AdomdConnection conn =
                new Microsoft.AnalysisServices.AdomdClient.AdomdConnection(connectionString);

            conn.Open();

            CubeDef cubeDef = conn.Cubes[cubeName];

            if (cubeDef == null)
            {
                return null;
            }

            List<string> measureGroupsNames = new List<string>();
            foreach (Microsoft.AnalysisServices.AdomdClient.Measure measure in cubeDef.Measures)
            {
                string measureGroupName = measure.Properties["MEASUREGROUP_NAME"].Value.ToString();
                if (!measureGroupsNames.Contains(measureGroupName))
                {
                    measureGroupsNames.Add(measureGroupName);
                }
            }

            return measureGroupsNames;
        }

        [OperationContract]
        public Dictionary<string, List<string>> LoadMeasureGroupDimensions(string connectionString, string cubeName, string catalogName)
        {
            Microsoft.AnalysisServices.AdomdClient.AdomdConnection conn =
                 new Microsoft.AnalysisServices.AdomdClient.AdomdConnection(connectionString);

            conn.Open();

            AdomdRestrictionCollection restrictions = new AdomdRestrictionCollection();
            restrictions.Add("CATALOG_NAME", catalogName);
            restrictions.Add("CUBE_NAME", cubeName);

            DataSet result = conn.GetSchemaDataSet("MDSCHEMA_MEASUREGROUP_DIMENSIONS", restrictions);
            Dictionary<string, List<string>> groupDimensions = new Dictionary<string, List<string>>();

            foreach (DataRow row in result.Tables[0].Rows)
            {
                string groupName = (string)row["MEASUREGROUP_NAME"];
                string dimensionUniqueName = (string)row["DIMENSION_UNIQUE_NAME"];

                List<string> dimensions = null;
                if (groupDimensions.TryGetValue(groupName, out dimensions))
                {
                    dimensions.Add(dimensionUniqueName);
                }
                else
                {
                    groupDimensions.Add(groupName, new List<string> { dimensionUniqueName });
                }
            }

            return groupDimensions;
        }

        [OperationContract]
        public ObservableCollection<Member> LoadLevelMembers(string connectionString, string cubeName, string levelUniqueName)
        {
            Microsoft.AnalysisServices.AdomdClient.AdomdConnection conn =
                new Microsoft.AnalysisServices.AdomdClient.AdomdConnection(connectionString);
            conn.Open();

            CubeDef cubeDef = conn.Cubes[cubeName];

            if (cubeDef == null)
            {
                return null;
            }

            MatchCollection matches = Regex.Matches(levelUniqueName, @"(\[.+?\])");
            if (matches.Count != 3)
            {
                throw new ArgumentException("Level unique name does not follow the expected format [Dimension].[Hierarchy].[Level]", "levelUniqueName");
            }

            string dimensionName = matches[0].Value.Replace("[", "").Replace("]", "");
            Microsoft.AnalysisServices.AdomdClient.Dimension dimension = cubeDef.Dimensions[dimensionName];

            if (dimension == null)
            {
                return null;
            }

            string hierarchyName = matches[1].Value.Replace("[", "").Replace("]", "");
            Microsoft.AnalysisServices.AdomdClient.Hierarchy hierarchy = dimension.Hierarchies[hierarchyName];

            if (hierarchy == null)
            {
                return null;
            }

            string levelName = matches[2].Value.Replace("[", "").Replace("]", "");
            Microsoft.AnalysisServices.AdomdClient.Level adomdLevel = hierarchy.Levels[levelName];

            if (adomdLevel == null)
            {
                return null;
            }

            MemberCollection levelMembers = adomdLevel.GetMembers();
            ObservableCollection<AdomdMember> members = new ObservableCollection<Member>();

            foreach (Microsoft.AnalysisServices.AdomdClient.Member memberData in levelMembers)
            {
                string parentName = memberData.Parent == null ? string.Empty : memberData.Parent.Name;

                AdomdMember member =
                    new AdomdMember()
                    {
                        Name = memberData.Name,
                        UniqueName = memberData.UniqueName,
                        Caption = memberData.Caption,
                        ParentUniqueName = parentName,
                        LevelUniqueName = memberData.LevelName,
                        LevelNumber = (uint)memberData.LevelDepth,
                        ChildrenCardinality = (int)memberData.ChildCount
                    };

                members.Add(member);
            }

            return members;
        }

        [OperationContract]
        public ObservableCollection<Member> LoadFilterMemberMembers(string connectionString, string cubeName, string parentMemberUniqueName, int levelNumber)
        {
            Microsoft.AnalysisServices.AdomdClient.AdomdConnection conn =
                new Microsoft.AnalysisServices.AdomdClient.AdomdConnection(connectionString);
            conn.Open();

            CubeDef cubeDef = conn.Cubes[cubeName];

            if (cubeDef == null)
            {
                return null;
            }

            MatchCollection matches = Regex.Matches(parentMemberUniqueName, @"(\[.+?\])");
            if (matches.Count < 2)
            {
                throw new ArgumentException("Level unique name does not follow the expected format [Dimension].[Hierarchy].[Member]", "hierarchyUniqueName");
            }

            string dimensionName = matches[0].Value.Replace("[", "").Replace("]", "");
            Microsoft.AnalysisServices.AdomdClient.Dimension dimension = cubeDef.Dimensions[dimensionName];

            if (dimension == null)
            {
                return null;
            }

            string hierarchyName = matches[1].Value.Replace("[", "").Replace("]", "");
            Microsoft.AnalysisServices.AdomdClient.Hierarchy hierarchy = dimension.Hierarchies[hierarchyName];

            if (hierarchy == null)
            {
                return null;
            }

            Microsoft.AnalysisServices.AdomdClient.Level adomdLevel = hierarchy.Levels[levelNumber];

            if (adomdLevel == null)
            {
                return null;
            }

            MemberCollection levelMembers = adomdLevel.GetMembers();
            ObservableCollection<AdomdMember> members = new ObservableCollection<Member>();

            foreach (Microsoft.AnalysisServices.AdomdClient.Member memberData in levelMembers)
            {
                if (memberData.Parent.UniqueName != parentMemberUniqueName)
                {
                    continue;
                }

                string parentName = memberData.Parent == null ? string.Empty : memberData.Parent.Name;

                AdomdMember member =
                    new AdomdMember()
                    {
                        Name = memberData.Name,
                        UniqueName = memberData.UniqueName,
                        Caption = memberData.Caption,
                        ParentUniqueName = parentName,
                        LevelUniqueName = memberData.LevelName,
                        LevelNumber = (uint)memberData.LevelDepth,
                        ChildrenCardinality = (int)memberData.ChildCount
                    };

                members.Add(member);
            }

            return members;
        }

        [OperationContract]
        public GridData ExecuteCommand(string connectionString, string query)
        {
            Microsoft.AnalysisServices.AdomdClient.AdomdConnection conn =
                new Microsoft.AnalysisServices.AdomdClient.AdomdConnection(connectionString);

            conn.Open();

            Microsoft.AnalysisServices.AdomdClient.AdomdCommand command =
                new Microsoft.AnalysisServices.AdomdClient.AdomdCommand(query, conn);

            Microsoft.AnalysisServices.AdomdClient.CellSet cellSet = command.ExecuteCellSet();

            GridData gridData = new GridData { Axes = new List<AxisData>() };
            foreach (Microsoft.AnalysisServices.AdomdClient.Axis axis in cellSet.Axes)
            {
                AxisData axisData = new AxisData { Positions = new List<PositionData>() };

                foreach (Microsoft.AnalysisServices.AdomdClient.Position position in axis.Positions)
                {
                    PositionData positionData = new PositionData
                    {
                        Ordinal = position.Ordinal,
                        Members = new List<MemberData>()
                    };

                    foreach (Microsoft.AnalysisServices.AdomdClient.Member member in position.Members)
                    {
                        MemberData memberData = new MemberData
                        {
                            Caption = member.Caption,
                            ChildCount = (int)member.ChildCount,
                            LevelDepth = member.LevelDepth,
                            LevelName = member.LevelName,
                            Name = member.Name,
                            ParentUniqueName = member.Parent == null ? string.Empty : member.Parent.UniqueName,
                            UniqueName = member.UniqueName,
                            DrilledDown = member.DrilledDown,
                            ParentSameAsPrevious = member.ParentSameAsPrevious
                        };

                        positionData.Members.Add(memberData);
                    }

                    axisData.Positions.Add(positionData);
                }

                gridData.Axes.Add(axisData);
            }

            gridData.Cells = new List<CellData>();

            foreach (Microsoft.AnalysisServices.AdomdClient.Cell cell in cellSet.Cells)
            {
                CellData cellData = new CellData
                {
                    FormattedValue = cell.FormattedValue,
                    Value = cell.Value
                };

                gridData.Cells.Add(cellData);
            }

            return gridData;
        }
    }
}

