﻿namespace CustomDataProvider
{
    using System;
    using System.Collections.Generic;
    using System.Collections.ObjectModel;
    using System.ComponentModel;
    using CustomDataProvider.AnalysisAccess;
    using Infragistics.Olap;
    using Infragistics.Olap.Data;
    using Infragistics.Olap.Data.Base;
    using Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008;
    using Infragistics.Olap.Xmla;

    using AdomdMember = CustomDataProvider.AnalysisAccess.AdomdMember;
    using Cube = Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Cube;
    using Dimension = Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Dimension;
    using LoadCubesCompletedEventArgs = Infragistics.Olap.LoadCubesCompletedEventArgs;
    using LoadSchemaCompletedEventArgs = Infragistics.Olap.LoadSchemaCompletedEventArgs;
    using Measure = Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Measure;
    using Member = Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Member;

    public class AdomdDataProvider : XmlaDataProvider
    {
        private AdomdConnection _connection;

        public AdomdDataProvider(IOlapConnection connection) : base(connection)
        {
            this._connection = connection as AdomdConnection;
        } 

        #region Properties

        public override IConnectionSettings ConnectionSettings
        {
            get
            {
                return this._connection.ConnectionSettings;
            }
        }

        protected string ConnectionString
        {
            get
            {
                AdomdConnectionSettings adomdConnectionSettings = this.ConnectionSettings as AdomdConnectionSettings;
                if (adomdConnectionSettings != null)
                {
                    return adomdConnectionSettings.ConnectionString;
                }

                return null;
            }
        }

        #endregion // Properties
   
        public override void LoadSchemaAsync()
        {                 
            AdomdConnectorClient client = new AdomdConnectorClient();
            client.LoadSchemaCompleted += (s, args) =>
            {
                Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.OlapSchema schema =
                new Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.OlapSchema();

                if (args.Error == null && !args.Cancelled)
                {
                    string databaseName = args.Result;
                    AddDatabase(schema, new Database(databaseName, databaseName, databaseName, databaseName));
                }

                this.OnLoadSchemaCompleted(new LoadSchemaCompletedEventArgs(schema, args.Error, args.Cancelled, schema));
            };

            client.LoadSchemaAsync(this.ConnectionString); 
        }

        public override void LoadCubesAsync(string catalogName)
        {
            AdomdConnectorClient client = new AdomdConnectorClient();

            client.LoadCubesCompleted += (s, args) =>
            {
                Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.OlapSchema schema =
                    new Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.OlapSchema();

                if (args.Error == null && !args.Cancelled)
                {
                    foreach (AdomdCube cube in args.Result)
                    {
                        if (cube.CubeType == "Cube")
                        {
                            AddCube(
                                schema,
                                new Cube(
                                    cube.CubeName,
                                    cube.Caption,
                                    cube.Description,
                                    CubeType.Cube,
                                    cube.CatalogName,
                                    cube.LastDataUpdate,
                                    cube.LastDataUpdate));
                        }
                    }
                }

                this.OnLoadCubesCompleted(new LoadCubesCompletedEventArgs(schema, args.Error, args.Cancelled, schema));
            };

            client.LoadCubesAsync(this.ConnectionString); 
        }

        public override void LoadDimensionsAsync(ICube cube)
        {
            AdomdConnectorClient client = new AdomdConnectorClient();
            client.LoadDimensionsCompleted += (s, args) =>
            {
                if (args.Error == null && !args.Cancelled)
                {
                    foreach (AdomdDimension dimension in args.Result)
                    {
                        DimensionType dimensionType =
                            (DimensionType)Enum.Parse(typeof(DimensionType), dimension.DimensionType.ToString(), true);

                        Dimension currentDimension = new Dimension(
                            dimension.Name,
                            dimension.UniqueName,
                            dimension.Caption,
                            dimension.Description,
                            dimensionType);

                        foreach (AdomdHierarchy adomdHierarchy in dimension.Hierarchies)
                        {
                            Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Hierarchy hierarchy =
                                new Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Hierarchy(
                                    adomdHierarchy.Name,
                                    adomdHierarchy.UniqueName,
                                    adomdHierarchy.Caption,
                                    adomdHierarchy.Description,
                                    adomdHierarchy.DefaultMember,
                                    adomdHierarchy.DefaultMember,
                                    currentDimension,
                                    string.Empty,
                                    HierarchyOrigin.SystemEnabled);

                            foreach (AdomdLevel adomdLevel in adomdHierarchy.Levels)
                            {
                                Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Level level =
                            new Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Level(
                                adomdLevel.Name,
                                adomdLevel.Caption,
                                adomdLevel.UniqueName,
                                adomdLevel.Description,
                                (int)adomdLevel.Number,
                                hierarchy);

                                AddLevel(hierarchy, level);
                            }

                            AddHierarchy(currentDimension, hierarchy);
                        }

                        AddDimension((Cube)cube, currentDimension);
                    }
                }

                this.OnLoadDimensionsCompleted(new AsyncCompletedEventArgs(args.Error, args.Cancelled, cube));            
            };

            client.LoadDimensionsAsync(this.ConnectionString, cube.Name);
        }

        public override void LoadMeasuresAsync(ICube cube)
        {
            AdomdConnectorClient client = new AdomdConnectorClient();
            client.LoadMeasuresCompleted += (s, args) =>
            {
                if (args.Error == null && !args.Cancelled)
                {
                    foreach (AdomdMeasure measure in args.Result)
                    {
                        MeasureAggregator aggregator =
                            (MeasureAggregator)Enum.Parse(typeof(MeasureAggregator), measure.Aggregator, true);

                        AddMeasure(
                            (Cube)cube,
                            new Measure(measure.Name, measure.UniqueName, measure.Caption, measure.Description, measure.DefaultFormatString, measure.MeasureGroupName, aggregator));
                    }
                }

                this.OnLoadMeasuresCompleted(new AsyncCompletedEventArgs(args.Error, args.Cancelled, args.UserState));
            };

            client.LoadMeasuresAsync(this.ConnectionString, cube.Name, cube);
        }

        public override void LoadMembersAsync(ICube cube, IFilterMember filterMember)
        {
            AdomdConnectorClient client = new AdomdConnectorClient();
            client.LoadFilterMemberMembersCompleted += (s, args) =>
            {
                if (args.Error == null && !args.Cancelled)
                {
                    Member prentMember = filterMember.Member as Member;
                    ClearMemberChildren(prentMember);

                    foreach (AdomdMember member in args.Result)
                    {
                        Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Member currentMember =
                        new Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Member(
                            member.Name,
                            member.UniqueName,
                            member.Caption,
                            member.ParentUniqueName,
                            member.LevelUniqueName,
                            (int)member.LevelNumber,
                            filterMember.Member.ParentLevel.ParentHierarchy.Levels[(int)member.LevelNumber],
                            (int)member.ChildrenCardinality,
                            false,
                            false,
                            false);

                        AddMember(prentMember, currentMember);
                    }
                }

                this.OnLoadMembersCompleted(new AsyncCompletedEventArgs(args.Error, args.Cancelled, filterMember));
            };

            client.LoadFilterMemberMembersAsync(this.ConnectionString, cube.Name, filterMember.Member.UniqueName, filterMember.Member.ParentLevel.Depth + 1);            
        }

        public override void LoadMembersAsync(ICube cube, ILevel level)
        {
            AdomdConnectorClient client = new AdomdConnectorClient();
            client.LoadLevelMembersCompleted += (s, args) =>
            {
                if (args.Error == null && !args.Cancelled)
                {
                    ClearMemberChildren(level as Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Level);

                    foreach (AdomdMember member in args.Result)
                    {
                        Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Member currentMember =
                        new Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Member(
                            member.Name,
                            member.UniqueName,
                            member.Caption,
                            member.ParentUniqueName,
                            member.LevelUniqueName,
                            (int)member.LevelNumber,
                            level.ParentHierarchy.Levels[(int)member.LevelNumber],
                            (int)member.ChildrenCardinality,
                            false,
                            false,
                            false);

                        AddMember(level as Infragistics.Olap.Data.Xmla.SqlAnalysisServices2008.Level, currentMember);
                    }
                }

                this.OnLoadMembersCompleted(new AsyncCompletedEventArgs(args.Error, args.Cancelled, level));
            };

            client.LoadLevelMembersAsync(this.ConnectionString, cube.Name, level.UniqueName);
        }

        public override void LoadMeasureGroupsAsync(ICube cube)
        {
            AdomdConnectorClient client = new AdomdConnectorClient();
            client.LoadMeasureGroupsCompleted += (s, args) =>
            {
                if (args.Error == null && !args.Cancelled)
                {
                    if (args.Result.Count > 0)
                    {
                        AddMeasureGroup(
                            (Cube)cube, MeasureGroup.AllMeasureGroup);
                    }

                    foreach (string measureGroup in args.Result)
                    {
                        AddMeasureGroup(
                            (Cube)cube,
                            new MeasureGroup(measureGroup, measureGroup, measureGroup));
                    }
                }

                this.OnLoadMeasureGroupsCompleted(new AsyncCompletedEventArgs(args.Error, args.Cancelled, cube.MeasureGroups));
            };

            client.LoadMeasureGroupsAsync(this.ConnectionString, cube.Name, cube);
        }

        public override void LoadMeasureGroupDimensionsAsync(ICube cube)
        {
            AdomdConnectorClient client = new AdomdConnectorClient();
            client.LoadMeasureGroupDimensionsCompleted += (s, args) =>
            {
                if (args.Error == null && !args.Cancelled)
                {
                    foreach (KeyValuePair<string, ObservableCollection<string>> keyValuePair in args.Result)
                    {
                        foreach (string dimensionName in keyValuePair.Value)
                        {
                            AddMeasureGroupDimensions((Cube)cube, keyValuePair.Key, dimensionName);                            
                        }
                    }
                }

                this.OnLoadMeasureGroupDimensionsCompleted(new AsyncCompletedEventArgs(args.Error, args.Cancelled, cube.MeasureGroups));
            };

            client.LoadMeasureGroupDimensionsAsync(this.ConnectionString, cube.Name, ((Cube)cube).CatalogName);            
        }
    }    
}
